//+------------------------------------------------------------------+
//|                                       Linear Regression Line.mq4 |
//|                                                      MQL Service |
//|                                           scripts@mqlservice.com |
//+------------------------------------------------------------------+
#property copyright "MQL Service"
#property link      "www.mqlservice.com"

#property indicator_chart_window
#property indicator_buffers 3
#property indicator_color1  LimeGreen
#property indicator_color2  Red
#property indicator_color3  Red
#property indicator_width1  2
#property indicator_width2  2
#property indicator_width3  2

//
//
//
//
//

extern string TimeFrame   = "Current time frame";
extern int    LrsPeriod   = 20;
extern int    Price       =  0;
extern bool   MultiColor  = true;

//
//
//
//
//

double lrs[];
double lrsDa[];
double lrsDb[];
double slope[];
double pr[];

//
//
//
//
//

string indicatorFileName;
bool   returnBars;
bool   calculateValue;
int    timeFrame;

//
//
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

int init()
{
	IndicatorBuffers(5);
	SetIndexBuffer(0,lrs);SetIndexStyle(0,DRAW_LINE); SetIndexDrawBegin(0,LrsPeriod+2);
	SetIndexBuffer(1,lrsDa);
   SetIndexBuffer(2,lrsDb);
   SetIndexBuffer(3,slope);
   SetIndexBuffer(4,pr);
   IndicatorDigits(MarketInfo(Symbol(), MODE_DIGITS)+4);
   LrsPeriod = MathMax(LrsPeriod,1);
   
      //
      //
      //
      //
      //
      
         indicatorFileName = WindowExpertName();
         calculateValue    = TimeFrame=="calculateValue"; if (calculateValue) { return(0); }
         returnBars        = TimeFrame=="returnBars";     if (returnBars)     { return(0); }
         timeFrame         = stringToTimeFrame(TimeFrame);
      
      //
      //
      //
      //
      //
               
   IndicatorShortName(timeFrameToString(timeFrame)+ "  LinearRegression-Real ("+DoubleToStr(LrsPeriod,2)+")");
   return(0);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//

int deinit(){return(0);}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//

int start()
{
	int counted_bars=IndicatorCounted();
   int i,j,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { lrs[0] = MathMin(limit+1,Bars-1); return(0); }
   
   //
   //
   //
   //
   //
   
   if (calculateValue || timeFrame == Period())
   {
   //
   //
   //
   //
   //
   
   if (!calculateValue && MultiColor && slope[limit]==-1) CleanPoint(limit,lrsDa,lrsDb);
   
	for(i=limit; i>=0; i--)
   {
	  double	sumx  = 0;
	  double	sumy  = 0;
	  double	sumxy = 0;
	  double	sumx2 = 0;
	  double	sumy2 = 0;
	         pr[i] = iMA(NULL,0,1,0,MODE_EMA,Price,i);
	  
	  //
	  //
	  //
	  //
	  //
	  
		for (j = 0; j <= LrsPeriod-1; j++)
		{ 
			sumx  = sumx  + j;                                                 
			sumy  = sumy  + pr[i+j];                                     
			sumxy = sumxy + j * pr[i+j];                                 
			sumx2 = sumx2 + j * j;                                                 
			sumy2 = sumy2 + pr[i+j] * pr[i+j];                 
		}
		
		//
      //
      //
      //
      //
      
		double lrslope = (LrsPeriod * sumxy - sumx * sumy) / (LrsPeriod * sumx2 - sumx * sumx);
		double yint = (sumy - lrslope * sumx) / LrsPeriod; 
		double r = (LrsPeriod * sumxy - sumx * sumy) / MathSqrt((LrsPeriod * sumx2 - sumx * sumx) * (LrsPeriod * sumy2 - sumy * sumy));

		lrs[i]   = lrslope*0+yint;
		lrsDa[i] = EMPTY_VALUE;
      lrsDb[i] = EMPTY_VALUE;
      slope[i] = slope[i+1];

      if (lrs[i]<lrs[i+1]) slope[i] = -1;
      if (lrs[i]>lrs[i+1]) slope[i] =  1;
      if (!calculateValue && MultiColor && slope[i] == -1) PlotPoint(i,lrsDa,lrsDb,lrs);

	}
	return(0);
}

   //
   //
   //
   //
   //
   
   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   if (slope[limit]==-1) CleanPoint(limit,lrsDa,lrsDb);
   for (i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         slope[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",LrsPeriod,Price,3,y);
         lrs[i]   = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",LrsPeriod,Price,0,y);
         lrsDa[i] = EMPTY_VALUE;
         lrsDb[i] = EMPTY_VALUE;

         //
         //
         //
         //
         //

   }
   if (MultiColor) for (i=limit;i>=0;i--) if (slope[i]==-1) PlotPoint(i,lrsDa,lrsDb,lrs);
   return(0);      
}



//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

//
//
//
//
//

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (first[i+1] == EMPTY_VALUE)
      {
         if (first[i+2] == EMPTY_VALUE) {
                first[i]   = from[i];
                first[i+1] = from[i+1];
                second[i]  = EMPTY_VALUE;
            }
         else {
                second[i]   =  from[i];
                second[i+1] =  from[i+1];
                first[i]    = EMPTY_VALUE;
            }
      }
   else
      {
         first[i]  = from[i];
         second[i] = EMPTY_VALUE;
      }
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}

